@@ -16,10 +16,12 @@
 
 
 import errno
+import fcntl
 import inspect
 import logging
 import logging.handlers
 import os
+import select
 import subprocess
 import sys
 import traceback
@@ -138,7 +140,9 @@ class Application(object):
             else:
                 self.output = sys.stdout
 
+            self.setup()
             self.process_args(args)
+            self.cleanup()
         except AppException, e:
             log(traceback.format_exc())
             stderr.write('ERROR: %s\n' % str(e))
@@ -159,7 +163,10 @@ class Application(object):
             sys.exit(1)
         except OSError, e: # pragma: no cover
             log(traceback.format_exc())
-            stderr.write('ERROR: %s\n' % str(e))
+            if hasattr(e, 'filename') and e.filename:
+                stderr.write('ERROR: %s: %s\n' % (e.filename, e.strerror))
+            else:
+                stderr.write('ERROR: %s\n' % e.strerror)
             sys.exit(1)
         except BaseException, e: # pragma: no cover
             log(traceback.format_exc())
@@ -303,6 +310,26 @@ class Application(object):
                                          arg_synopsis=self.arg_synopsis,
                                          cmd_synopsis=self.cmd_synopsis)
 
+    def setup(self):
+        '''Prepare for process_args.
+        
+        This method is called just before process_args. By default it
+        does nothing, but subclasses may override it with a suitable
+        implementation. This is easier than overriding process_args
+        itself.
+        
+        '''
+
+    def cleanup(self):
+        '''Clean up after process_args.
+        
+        This method is called just after process_args. By default it
+        does nothing, but subclasses may override it with a suitable
+        implementation. This is easier than overriding process_args
+        itself.
+        
+        '''
+
     def process_args(self, args):
         '''Process command line non-option arguments.
         
@@ -385,8 +412,11 @@ class Application(object):
         
         '''
         
-    def runcmd(self, argv, ignore_fail=False, *args, **kwargs):
-        '''Run external command.
+    def runcmd(self, argv, *args, **kwargs):
+        '''Run external command or pipeline.
+
+        Example: ``runcmd(['grep', 'foo'], ['wc', '-l'], 
+                          feed_stdin='foo\nbar\n')``
 
         Return the standard output of the command.
         
@@ -396,6 +426,12 @@ class Application(object):
         
         '''
 
+        if 'ignore_fail' in kwargs:
+            ignore_fail = kwargs['ignore_fail']
+            del kwargs['ignore_fail']
+        else:
+            ignore_fail = False
+
         exit, out, err = self.runcmd_unchecked(argv, *args, **kwargs)
         if exit != 0:
             msg = 'Command failed: %s\n%s' % (' '.join(argv), err)
@@ -406,25 +442,144 @@ class Application(object):
                 raise cliapp.AppException(msg)
         return out
         
-    def runcmd_unchecked(self, argv, feed_stdin=None, ignore_fail=False, *args, 
-                         **kwargs):
-        '''Run external command.
+    def runcmd_unchecked(self, argv, *argvs, **kwargs):
+        '''Run external command or pipeline.
 
         Return the exit code, and contents of standard output and error
         of the command.
         
+        See also ``runcmd``.
+        
         '''
 
-        logging.debug('run external command: %s' % ' '.join(argv))
-        if 'stdout' not in kwargs:
-            kwargs['stdout'] = subprocess.PIPE
-        if 'stderr' not in kwargs:
-            kwargs['stderr'] = subprocess.PIPE
-        if feed_stdin is not None and 'stdin' not in kwargs:
-            kwargs['stdin'] = subprocess.PIPE
-        p = subprocess.Popen(argv, *args, **kwargs)
-        out, err = p.communicate(feed_stdin)
-        return p.returncode, out, err
+        argvs = [argv] + list(argvs)
+        logging.debug('run external command: %s' % repr(argvs))
+
+        def pop_kwarg(name, default):
+            if name in kwargs:
+                value = kwargs[name]
+                del kwargs[name]
+                return value
+            else:
+                return default
+
+        feed_stdin = pop_kwarg('feed_stdin', '')
+        pipe_stdin = pop_kwarg('stdin', subprocess.PIPE)
+        pipe_stdout = pop_kwarg('stdout', subprocess.PIPE)
+        pipe_stderr = pop_kwarg('stderr', subprocess.PIPE)
+
+        try:
+            pipeline = self._build_pipeline(argvs,
+                                            pipe_stdin,
+                                            pipe_stdout,
+                                            pipe_stderr,
+                                            kwargs)
+            return self._run_pipeline(pipeline, feed_stdin, pipe_stdin,
+                                      pipe_stdout, pipe_stderr)
+        except OSError, e: # pragma: no cover
+            if e.errno == errno.ENOENT and e.filename is None:
+                e.filename = argv[0]
+                raise e
+            else:
+                raise
+
+    def _build_pipeline(self, argvs, pipe_stdin, pipe_stdout, pipe_stderr,
+                        kwargs):
+        procs = []
+        for i, argv in enumerate(argvs):
+            if i == 0 and i == len(argvs) - 1:
+                stdin = pipe_stdin
+                stdout = pipe_stdout
+                stderr = pipe_stderr
+            elif i == 0:
+                stdin = pipe_stdin
+                stdout = subprocess.PIPE
+                stderr = pipe_stderr
+            elif i == len(argv) - 1:
+                stdin = procs[-1].stdout
+                stdout = pipe_stdout
+                stderr = pipe_stderr
+            else:
+                stdin = procs[-1].stdout
+                stdout = subprocess.PIPE
+                stderr = pipe_stderr
+            p = subprocess.Popen(argv, stdin=stdin, stdout=stdout, 
+                                 stderr=stderr, close_fds=True, **kwargs)
+            procs.append(p)
+
+        return procs
+
+    def _run_pipeline(self, procs, feed_stdin, pipe_stdin, pipe_stdout,
+                      pipe_stderr):
+        stdout_eof = False
+        stderr_eof = False
+        out = []
+        err = []
+        pos = 0
+        io_size = 1024
+        
+        def set_nonblocking(fd):
+            flags = fcntl.fcntl(fd, fcntl.F_GETFL, 0)
+            flags = flags | os.O_NONBLOCK
+            fcntl.fcntl(fd, fcntl.F_SETFL, flags)
+
+        if feed_stdin and pipe_stdin == subprocess.PIPE:
+            set_nonblocking(procs[0].stdin.fileno())
+        if pipe_stdout == subprocess.PIPE:
+            set_nonblocking(procs[-1].stdout.fileno())
+        if pipe_stderr == subprocess.PIPE:
+            set_nonblocking(procs[-1].stderr.fileno())
+        
+        def still_running():
+            for p in procs:
+                p.poll()
+            for p in procs:
+                if p.returncode is None:
+                    return True
+            if pipe_stdout == subprocess.PIPE and not stdout_eof:
+                return True
+            if pipe_stderr == subprocess.PIPE and not stderr_eof:
+                return True # pragma: no cover
+            return False
+
+        while still_running():
+            rlist = []
+            if not stdout_eof and pipe_stdout == subprocess.PIPE:
+                rlist.append(procs[-1].stdout)
+            if not stderr_eof and pipe_stderr == subprocess.PIPE:
+                rlist.append(procs[-1].stderr)
+            
+            wlist = []
+            if pipe_stdin == subprocess.PIPE and pos < len(feed_stdin):
+                wlist.append(procs[0].stdin)
+
+            if rlist or wlist:
+                r, w, x = select.select(rlist, wlist, [])
+            else:
+                r = w = [] # pragma: no cover
+
+            if procs[0].stdin in w and pos < len(feed_stdin):
+                data = feed_stdin[pos : pos+io_size]
+                procs[0].stdin.write(data)
+                pos += len(data)
+                if pos >= len(feed_stdin):
+                    procs[0].stdin.close()
+
+            if procs[-1].stdout in r:
+                data = procs[-1].stdout.read(io_size)
+                if data:
+                    out.append(data)
+                else:
+                    stdout_eof = True
+
+            if procs[-1].stderr in r:
+                data = procs[-1].stderr.read(io_size)
+                if data:
+                    err.append(data)
+                else:
+                    stderr_eof = True
+
+        return procs[-1].returncode, ''.join(out), ''.join(err)
 
     def _vmrss(self): # pragma: no cover
         '''Return current resident memory use, in KiB.'''
